@@ -195,6 +195,34 @@ class SimpleArcTest(CoverageTest):
         )
         assert self.stdout() == "5\n"
 
+    def test_bug_576(self) -> None:
+        if env.PYBEHAVIOR.pep626:
+            branchz = "34 36 89 8B"
+            branchz_missing = ""
+        else:
+            branchz = "34 38 89 8D"
+            branchz_missing = "38 8D"
+
+        self.check_coverage("""\
+            foo = True
+
+            if foo == True:
+                foo = False
+            else:  # pragma: no cover
+                pass
+
+            if foo == False:
+                foo = True
+            else:  # pragma: no cover
+                None
+
+            print("Done")
+            """,
+            branchz=branchz,
+            branchz_missing=branchz_missing,
+        )
+        assert self.stdout() == "Done\n"
+
 
 class WithTest(CoverageTest):
     """Arc-measuring tests involving context managers."""
@@ -2101,6 +2129,29 @@ class AsyncTest(CoverageTest):
             branchz_missing="29 38 45 56 5. 9A 9.",
         )
 
+    @pytest.mark.skipif(env.PYVERSION[:2] == (3, 9), reason="not sure why it isn't fixed on 3.9")
+    def test_bug_1999(self) -> None:
+        self.check_coverage("""\
+            import asyncio
+
+            async def async_range(number):
+                for n in range(number):
+                    yield n
+
+            async def do_something(number):
+                try:
+                    async for n in async_range(number):
+                        print(n)
+                finally:
+                    print("Done.")
+
+            asyncio.run(do_something(3))
+            """,
+            branchz="4-3 45 9A 9C",
+            branchz_missing="",
+        )
+        assert self.stdout() == "0\n1\n2\nDone.\n"
+
 
 class AnnotationTest(CoverageTest):
     """Tests using type annotations."""
@@ -2132,9 +2183,15 @@ class ExcludeTest(CoverageTest):
             e = 7
             if e:#\tpragma:\tno branch
                 f = 9
+            import typing
+            if typing.TYPE_CHECKING:    # only for mypy
+                g = 12
+            else:
+                h = 14
             """,
-            lines=[1,2,3,4,5,6,7,8,9],
-            branchz="23 24 56 57 89 8.",
+            lines=[1,2,3,4,5,6,7,8,9,10,14],
+            missing="",
+            branchz="23 24 56 57 89 8A BC BE",
             branchz_missing="",
         )
 
