@@ -7,10 +7,12 @@ from __future__ import annotations
 
 import csv
 import glob
+import itertools
 import os
 import os.path
 import platform
 import re
+import signal
 import stat
 import sys
 import textwrap
@@ -270,7 +272,7 @@ class ProcessTest(CoverageTest):
 
         self.run_command("coverage run fleeting.py")
         os.remove("fleeting.py")
-        out = self.run_command("coverage html -d htmlcov")
+        out = self.run_command("coverage html -d htmlcov", status=1)
         assert re.search("No source for code: '.*fleeting.py'", out)
         assert "Traceback" not in out
 
@@ -282,10 +284,9 @@ class ProcessTest(CoverageTest):
 
         self.run_command("coverage run fleeting")
         os.remove("fleeting")
-        status, out = self.run_command_status("coverage html -d htmlcov")
+        out = self.run_command("coverage html -d htmlcov", status=1)
         assert re.search("No source for code: '.*fleeting'", out)
         assert "Traceback" not in out
-        assert status == 1
 
     def test_running_missing_file(self) -> None:
         status, out = self.run_command_status("coverage run xyzzy.py")
@@ -310,8 +311,8 @@ class ProcessTest(CoverageTest):
 
         # The important thing is for "coverage run" and "python" to report the
         # same traceback.
-        status, out = self.run_command_status("coverage run throw.py")
-        out2 = self.run_command("python throw.py")
+        out = self.run_command("coverage run throw.py", status=1)
+        out2 = self.run_command("python throw.py", status=1)
         if env.PYPY:
             # PyPy has an extra frame in the traceback for some reason
             out2 = re_lines_text("toplevel", out2, match=False)
@@ -322,7 +323,6 @@ class ProcessTest(CoverageTest):
         msg = f'File "{re.escape(path)}", line 8, in f2'
         assert re.search(msg, out)
         assert 'raise MyException("hey!")' in out
-        assert status == 1
 
     def test_code_exits(self) -> None:
         self.make_file("exit.py", """\
@@ -410,7 +410,7 @@ class ProcessTest(CoverageTest):
         data.read()
         assert line_counts(data)["fork.py"] == total_lines
 
-        debug_text = Path("debug.out").read_text()
+        debug_text = Path("debug.out").read_text(encoding="utf-8")
         ppid = pids["parent"]
         cpid = pids["child"]
         assert ppid != cpid
@@ -421,6 +421,46 @@ class ProcessTest(CoverageTest):
         reported_pids = {line.split(".")[0] for line in debug_text.splitlines()}
         assert len(reported_pids) == 2
 
+    @pytest.mark.skipif(not hasattr(os, "fork"), reason="Can't test os.fork, it doesn't exist.")
+    @pytest.mark.parametrize("patch", [False, True])
+    def test_os_exit(self, patch: bool) -> None:
+        self.make_file("forky.py", """\
+            import os
+            import tempfile
+            import time
+
+            complete_file = tempfile.mkstemp()[1]
+            pid = os.fork()
+            if pid:
+                while pid: # 3.9 wouldn't count "while True": change this. PYVERSION
+                    with open(complete_file, encoding="ascii") as f:
+                        data = f.read()
+                    if "Complete" in data:
+                        break
+                    time.sleep(.02)
+                os.remove(complete_file)
+            else:
+                time.sleep(.1)
+                with open(complete_file, mode="w", encoding="ascii") as f:
+                    f.write("Complete")
+                os._exit(0)
+                # if this exists, then we broke os._exit completely
+                open("impossible.txt", mode="w")
+            """)
+        total_lines = 17    # don't count the last impossible.txt line
+        if patch:
+            self.make_file(".coveragerc", "[run]\npatch = _exit\n")
+        self.run_command("coverage run -p forky.py")
+        self.run_command("coverage combine")
+        data = coverage.CoverageData()
+        data.read()
+        seen = line_counts(data)["forky.py"]
+        if patch:
+            assert seen == total_lines
+        else:
+            assert seen < total_lines
+        self.assert_doesnt_exist("impossible.txt")
+
     def test_warnings_during_reporting(self) -> None:
         # While fixing issue #224, the warnings were being printed far too
         # often.  Make sure they're not any more.
@@ -446,13 +486,13 @@ class ProcessTest(CoverageTest):
         # Note: the name of the function can't have "warning" in it, or the
         # absolute path of the file will have "warning" in it, and an assertion
         # will fail.
-        out = self.run_command("coverage run i_dont_exist.py")
+        out = self.run_command("coverage run i_dont_exist.py", status=1)
         path = python_reported_file('i_dont_exist.py')
         assert f"No file to run: '{path}'" in out
         assert "warning" not in out
         assert "Exception" not in out
 
-        out = self.run_command("coverage run -m no_such_module")
+        out = self.run_command("coverage run -m no_such_module", status=1)
         assert (
             ("No module named no_such_module" in out) or
             ("No module named 'no_such_module'" in out)
@@ -652,6 +692,35 @@ class ProcessTest(CoverageTest):
         out = self.run_command("python -m coverage")
         assert "Use 'coverage help' for help" in out
 
+    @pytest.mark.skipif(env.WINDOWS, reason="This test is not for Windows")
+    def test_save_signal_usr1(self) -> None:
+        self.assert_doesnt_exist(".coverage")
+        self.make_file("dummy_hello.py", """\
+            import os
+            import signal
+
+            print(f"Sending SIGUSR1 to myself")
+            os.kill(os.getpid(), signal.SIGUSR1)
+            os.kill(os.getpid(), signal.SIGKILL)
+
+            print("Done and goodbye")
+            """)
+        out = self.run_command(
+            "coverage run --save-signal=USR1 dummy_hello.py",
+            status=-signal.SIGKILL,
+        )
+        # `startswith` because on Linux it also prints "Killed"
+        assert out.startswith("Sending SIGUSR1 to myself\nSaving coverage data...\n")
+        self.assert_exists(".coverage")
+        out = self.run_command("coverage report -m")
+        assert out == textwrap.dedent("""\
+            Name             Stmts   Miss  Cover   Missing
+            ----------------------------------------------
+            dummy_hello.py       6      2    67%   6-8
+            ----------------------------------------------
+            TOTAL                6      2    67%
+            """)
+
 
 TRY_EXECFILE = os.path.join(os.path.dirname(__file__), "modules/process_test/try_execfile.py")
 
@@ -670,14 +739,14 @@ class EnvironmentTest(CoverageTest):
         assert actual == expected
 
     def test_coverage_run_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("run_me.py", f.read())
         expected = self.run_command("python run_me.py")
         actual = self.run_command("coverage run run_me.py")
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_far_away_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("sub/overthere/prog.py", f.read())
         expected = self.run_command("python sub/overthere/prog.py")
         actual = self.run_command("coverage run sub/overthere/prog.py")
@@ -685,7 +754,7 @@ class EnvironmentTest(CoverageTest):
 
     @pytest.mark.skipif(not env.WINDOWS, reason="This is about Windows paths")
     def test_coverage_run_far_away_is_like_python_windows(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("sub/overthere/prog.py", f.read())
         expected = self.run_command("python sub\\overthere\\prog.py")
         actual = self.run_command("coverage run sub\\overthere\\prog.py")
@@ -698,7 +767,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dir_is_like_python_dir(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
 
         expected = self.run_command("python with_main")
@@ -706,7 +775,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_dir_no_init_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
 
         expected = self.run_command("python -m with_main")
@@ -714,7 +783,7 @@ class EnvironmentTest(CoverageTest):
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_dir_with_init_is_like_python(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
         self.make_file("with_main/__init__.py", "")
 
@@ -782,7 +851,7 @@ class EnvironmentTest(CoverageTest):
     def test_coverage_run_dashm_is_like_python_dashm_off_path(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/242
         self.make_file("sub/__init__.py", "")
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("sub/run_me.py", f.read())
 
         expected = self.run_command("python -m sub.run_me")
@@ -801,7 +870,7 @@ class EnvironmentTest(CoverageTest):
         # Test running coverage from a zip file itself.  Some environments
         # (windows?) zip up the coverage main to be used as the coverage
         # command.
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("run_me.py", f.read())
         expected = self.run_command("python run_me.py")
         cov_main = os.path.join(TESTS_DIR, "covmain.zip")
@@ -814,7 +883,7 @@ class EnvironmentTest(CoverageTest):
         reason="Windows gets this wrong: https://github.com/python/cpython/issues/131484",
     )
     def test_pythonsafepath(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("run_me.py", f.read())
         self.set_environ("PYTHONSAFEPATH", "1")
         expected = self.run_command("python run_me.py")
@@ -823,7 +892,7 @@ class EnvironmentTest(CoverageTest):
 
     @pytest.mark.skipif(env.PYVERSION < (3, 11), reason="PYTHONSAFEPATH is new in 3.11")
     def test_pythonsafepath_dashm_runme(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("run_me.py", f.read())
         self.set_environ("PYTHONSAFEPATH", "1")
         expected = self.run_command("python run_me.py")
@@ -832,12 +901,12 @@ class EnvironmentTest(CoverageTest):
 
     @pytest.mark.skipif(env.PYVERSION < (3, 11), reason="PYTHONSAFEPATH is new in 3.11")
     def test_pythonsafepath_dashm(self) -> None:
-        with open(TRY_EXECFILE) as f:
+        with open(TRY_EXECFILE, encoding="utf-8") as f:
             self.make_file("with_main/__main__.py", f.read())
 
         self.set_environ("PYTHONSAFEPATH", "1")
-        expected = self.run_command("python -m with_main")
-        actual = self.run_command("coverage run -m with_main")
+        expected = self.run_command("python -m with_main", status=1)
+        actual = self.run_command("coverage run -m with_main", status=1)
         assert re.search("No module named '?with_main'?", actual)
         assert re.search("No module named '?with_main'?", expected)
 
@@ -1002,10 +1071,8 @@ class ExcepthookTest(CoverageTest):
 
             raise RuntimeError('Error Outside')
             """)
-        cov_st, cov_out = self.run_command_status("coverage run excepthook_throw.py")
-        py_st, py_out = self.run_command_status("python excepthook_throw.py")
-        assert cov_st == py_st
-        assert cov_st == 1
+        cov_out = self.run_command("coverage run excepthook_throw.py", status=1)
+        py_out = self.run_command("python excepthook_throw.py", status=1)
         assert "in excepthook" in py_out
         assert cov_out == py_out
 
@@ -1025,8 +1092,9 @@ class AliasedCommandTest(CoverageTest):
         # "coverage2" doesn't work on py3
         assert sys.version_info[0] == 3    # Let us know when Python 4 is out...
         badcmd = "coverage2"
-        out = self.run_command(badcmd)
+        status, out = self.run_command_status(badcmd)
         assert "Code coverage for Python" not in out
+        assert status != 0
 
     def test_specific_alias_works(self) -> None:
         # "coverage-3.9" works on py3.9
@@ -1040,7 +1108,7 @@ class AliasedCommandTest(CoverageTest):
         "coverage-%d.%d" % sys.version_info[:2],
     ])
     def test_aliases_used_in_messages(self, cmd: str) -> None:
-        out = self.run_command(f"{cmd} foobar")
+        out = self.run_command(f"{cmd} foobar", status=1)
         assert "Unknown command: 'foobar'" in out
         assert f"Use '{cmd} help' for help" in out
 
@@ -1129,22 +1197,24 @@ class CoverageCoreTest(CoverageTest):
         has_ctracer = False
 
     def test_core_default(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.del_environ("COVERAGE_CORE")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run --debug=sys numbers.py")
         assert out.endswith("123 456\n")
         core = re_line(r" core:", out).strip()
-        # if env.PYBEHAVIOR.pep669:
-        #     assert core == "core: SysMonitor"
-        if self.has_ctracer:
+        warns = re_lines(r"\(no-ctracer\)", out)
+        if env.SYSMON_DEFAULT:
+            assert core == "core: SysMonitor"
+            assert not warns
+        elif self.has_ctracer:
             assert core == "core: CTracer"
+            assert not warns
         else:
             assert core == "core: PyTracer"
+            assert bool(warns) == env.CPYTHON
 
     @pytest.mark.skipif(not has_ctracer, reason="No CTracer to request")
     def test_core_request_ctrace(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.set_environ("COVERAGE_CORE", "ctrace")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run --debug=sys numbers.py")
@@ -1152,8 +1222,19 @@ class CoverageCoreTest(CoverageTest):
         core = re_line(r" core:", out).strip()
         assert core == "core: CTracer"
 
+    @pytest.mark.skipif(has_ctracer, reason="CTracer needs to be missing")
+    def test_core_request_ctrace_but_missing(self) -> None:
+        self.del_environ("COVERAGE_CORE")
+        self.make_file(".coveragerc", "[run]\ncore = ctrace\n")
+        self.make_file("numbers.py", "print(123, 456)")
+        out = self.run_command("coverage run --debug=sys,pybehave numbers.py")
+        assert out.endswith("123 456\n")
+        core = re_line(r" core:", out).strip()
+        assert core == "core: PyTracer"
+        warns = re_lines(r"\(no-ctracer\)", out)
+        assert bool(warns) == env.SHIPPING_WHEELS
+
     def test_core_request_pytrace(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.set_environ("COVERAGE_CORE", "pytrace")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run --debug=sys numbers.py")
@@ -1162,25 +1243,23 @@ class CoverageCoreTest(CoverageTest):
         assert core == "core: PyTracer"
 
     def test_core_request_sysmon(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.set_environ("COVERAGE_CORE", "sysmon")
         self.make_file("numbers.py", "print(123, 456)")
         out = self.run_command("coverage run --debug=sys numbers.py")
         assert out.endswith("123 456\n")
         core = re_line(r" core:", out).strip()
-        warns = re_lines(r"CoverageWarning: sys.monitoring isn't available", out)
+        warns = re_lines(r"\(no-sysmon\)", out)
         if env.PYBEHAVIOR.pep669:
             assert core == "core: SysMonitor"
             assert not warns
         else:
-            assert core in ("core: CTracer", "core: PyTracer")
+            assert core in ["core: CTracer", "core: PyTracer"]
             assert warns
 
     def test_core_request_nosuchcore(self) -> None:
-        self.del_environ("COVERAGE_TEST_CORES")
         self.set_environ("COVERAGE_CORE", "nosuchcore")
         self.make_file("numbers.py", "print(123, 456)")
-        out = self.run_command("coverage run numbers.py")
+        out = self.run_command("coverage run numbers.py", status=1)
         assert "Unknown core value: 'nosuchcore'\n" in out
         assert "123 456" not in out
 
@@ -1228,7 +1307,7 @@ class YankedDirectoryTest(CoverageTest):
 
     def test_removing_directory_with_error(self) -> None:
         self.make_file("bug806.py", self.BUG_806)
-        out = self.run_command("coverage run bug806.py")
+        out = self.run_command("coverage run bug806.py", status=1)
         path = python_reported_file('bug806.py')
         # Python 3.11 adds an extra line to the traceback.
         # Check that the lines we expect are there.
@@ -1242,6 +1321,7 @@ class YankedDirectoryTest(CoverageTest):
 
 
 @pytest.mark.skipif(env.METACOV, reason="Can't test subprocess pth file during metacoverage")
+@pytest.mark.xdist_group(name="needs_pth")
 class ProcessStartupTest(CoverageTest):
     """Test that we can measure coverage in subprocesses."""
 
@@ -1256,12 +1336,25 @@ class ProcessStartupTest(CoverageTest):
             """)
         # sub.py will write a few lines.
         self.make_file("sub.py", """\
-            f = open("out.txt", "w")
+            f = open("out.txt", "w", encoding="utf-8")
             f.write("Hello, world!\\n")
             f.close()
             """)
 
-    def test_subprocess_with_pth_files(self) -> None:
+    def test_patch_subprocess(self) -> None:
+        self.make_file(".coveragerc", """\
+            [run]
+            patch = subprocess
+            """)
+        self.run_command("coverage run main.py")
+        self.run_command("coverage combine")
+        self.assert_exists(".coverage")
+        data = coverage.CoverageData()
+        data.read()
+        assert line_counts(data)["main.py"] == 3
+        assert line_counts(data)["sub.py"] == 3
+
+    def test_subprocess_with_pth_files(self, _create_pth_file: None) -> None:
         # An existing data file should not be read when a subprocess gets
         # measured automatically.  Create the data file here with bogus data in
         # it.
@@ -1276,7 +1369,7 @@ class ProcessStartupTest(CoverageTest):
         self.set_environ("COVERAGE_PROCESS_START", "coverage.ini")
         import main             # pylint: disable=unused-import, import-error
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!\n"
 
         # Read the data from .coverage
@@ -1285,7 +1378,7 @@ class ProcessStartupTest(CoverageTest):
         data.read()
         assert line_counts(data)['sub.py'] == 3
 
-    def test_subprocess_with_pth_files_and_parallel(self) -> None:
+    def test_subprocess_with_pth_files_and_parallel(self, _create_pth_file: None) -> None:
         # https://github.com/nedbat/coveragepy/issues/492
         self.make_file("coverage.ini", """\
             [run]
@@ -1295,7 +1388,7 @@ class ProcessStartupTest(CoverageTest):
         self.set_environ("COVERAGE_PROCESS_START", "coverage.ini")
         self.run_command("coverage run main.py")
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!\n"
 
         self.run_command("coverage combine")
@@ -1315,6 +1408,72 @@ class ProcessStartupTest(CoverageTest):
         assert len(data_files) == 1, msg
 
 
+@pytest.mark.skipif(env.WINDOWS, reason="patch=execv isn't supported on Windows")
+@pytest.mark.xdist_group(name="needs_pth")
+class ExecvTest(CoverageTest):
+    """Test that we can measure coverage in subprocesses."""
+
+    @pytest.mark.parametrize("fname",
+        [base + suffix for base, suffix in itertools.product(
+            ["exec", "spawn"],
+            ["l", "le", "lp", "lpe", "v", "ve", "vp", "vpe"],
+        )]
+    )
+    def test_execv_patch(self, fname: str) -> None:
+        self.make_file(".coveragerc", """\
+            [run]
+            patch = subprocess, execv
+            """)
+        self.make_file("main.py", f"""\
+            import os, sys
+            print("In main")
+            args = []
+            if "spawn" in {fname!r}:
+                args.append(os.P_WAIT)
+            args.append(sys.executable)
+            prog_args = ["python", {os.path.abspath("other.py")!r}, "cat", "dog"]
+            if "l" in {fname!r}:
+                args.extend(prog_args)
+            else:
+                args.append(prog_args)
+            if {fname!r}.endswith("e"):
+                args.append({{"SUBVAR": "the-sub-var"}})
+            os.environ["MAINVAR"] = "the-main-var"
+            sys.stdout.flush()
+            os.{fname}(*args)
+            """)
+        self.make_file("other.py", """\
+            import os, sys
+            print(f"MAINVAR = {os.getenv('MAINVAR', 'none')}")
+            print(f"SUBVAR = {os.getenv('SUBVAR', 'none')}")
+            print(f"{sys.argv[1:] = }")
+            """)
+
+        out = self.run_command("coverage run main.py")
+        expected = "In main\n"
+        if fname.endswith("e"):
+            expected += "MAINVAR = none\n"
+            expected += "SUBVAR = the-sub-var\n"
+        else:
+            expected += "MAINVAR = the-main-var\n"
+            expected += "SUBVAR = none\n"
+        expected += "sys.argv[1:] = ['cat', 'dog']\n"
+        assert out == expected
+
+        self.run_command("coverage combine")
+        data = coverage.CoverageData()
+        data.read()
+
+        main_lines = 12
+        if "spawn" in fname:
+            main_lines += 1
+        if fname.endswith("e"):
+            main_lines += 1
+
+        assert line_counts(data)["main.py"] == main_lines
+        assert line_counts(data)["other.py"] == 4
+
+
 class ProcessStartupWithSourceTest(CoverageTest):
     """Show that we can configure {[run]source} during process-level coverage.
 
@@ -1332,7 +1491,14 @@ class ProcessStartupWithSourceTest(CoverageTest):
     @pytest.mark.parametrize("dashm", ["-m", ""])
     @pytest.mark.parametrize("package", ["pkg", ""])
     @pytest.mark.parametrize("source", ["main", "sub"])
-    def test_pth_and_source_work_together(self, dashm: str, package: str, source: str) -> None:
+    @pytest.mark.xdist_group(name="needs_pth")
+    def test_pth_and_source_work_together(
+        self,
+        dashm: str,
+        package: str,
+        source: str,
+        _create_pth_file: None,
+    ) -> None:
         """Run the test for a particular combination of factors.
 
         The arguments are all strings:
@@ -1368,7 +1534,7 @@ class ProcessStartupWithSourceTest(CoverageTest):
             self.make_file(path("__init__.py"), "")
         # sub.py will write a few lines.
         self.make_file(path("sub.py"), """\
-            f = open("out.txt", "w")
+            f = open("out.txt", "w", encoding="utf-8")
             f.write("Hello, world!")
             f.close()
             """)
@@ -1386,7 +1552,7 @@ class ProcessStartupWithSourceTest(CoverageTest):
 
         self.run_command(cmd)
 
-        with open("out.txt") as f:
+        with open("out.txt", encoding="utf-8") as f:
             assert f.read() == "Hello, world!"
 
         # Read the data from .coverage
